/******************************************************************************
 CBRegexQRefText.cc

	Copyright © 1998 by John Lindal. All rights reserved.

 ******************************************************************************/

#include "JXRegexHelpText.h"

const JCharacter* kRegexQRefName  = "CBRegexQRef";
const JCharacter* kRegexQRefTitle = "Regex Quick Reference";

const JCharacter* kRegexQRefText =

"<html>"
"<h3> Regex Quick Reference </h3>"

"<pre><font face=6x13>"

"<b>Matching</b>\n"
"\n"
"Characters and classes\n"
"\n"
"  .             Any single character\n"
"  []            Character class (see below)\n"
"\n"
"  \\t            Horizontal tab\n"
"  \\n            Newline\n"
"  \\r            Carriage return (cannot be typed in the Code Crusader text editor)\n"
"  \\f            Form feed (cannot be typed in the Code Crusader text editor)\n"
"  \\v            Vertical tab (cannot be typed in the Code Crusader text editor)\n"
"\n"
"  \\w            Word character:     [a-zA-Z0-9_]\n"
"  \\W            Non-word character: [^a-zA-Z0-9_]\n"
"\n"
"  \\s            Whitespace character:     [ \\t\\n\\r\\f\\v]\n"
"  \\S            Non-whitespace character: [^ \\t\\n\\r\\f\\v]\n"
"\n"
"  \\d            Decimal digit: [0-9]\n"
"  \\D            Non-digit:     [^0-9]\n"
"\n"
"Quantifiers\n"
"\n"
"  ?             Zero or one occurrence\n"
"  *             Zero or more occurrences\n"
"  +             One or more occurrences\n"
"  {min,max}     At least min and at most max occurrences\n"
"  {n}           Exactly n occurrences\n"
"  {min,}        At least min occurrences\n"
"\n"
"Other\n"
"\n"
"  x|y           Alternation: either x or y\n"
"  (...)         Grouping and subexpression capturing\n"
"\n"
"  ^             The beginning of a line\n"
"  $             The end of a line\n"
"\n"
"  \\&lt            The beginning of a word\n"
"  \\&gt            The end of a word\n"
"\n"
"Oddities\n"
"\n"
"  ()            Enclosing nothing, the null string\n"
"  {             If not followed by a number, matches itself.\n"
"\n"
"<b>Replacement</b>\n"
"\n"
"  \\0            The overall match\n"
"  \\N            The Nth parenthesized subexpression, counting '('s from the left\n"
"  \\-N           The Nth subexpression counting '('s from the right\n"
"\n"
"<b>Character Classes</b>\n"
"\n"
"  -             Between two characters, indicates a range.\n"
"                As the first character after any '^', the last\n"
"                character, or the second endpoint of a range,\n"
"                matches itself. ([.-.] can be used as the first\n"
"                endpoint of a range.)\n"
"\n"
"  ^             As the first character, complements the set.\n"
"                Otherwise, matches itself.\n"
"\n"
"  [             Matches itself, since classes don't nest.\n"
"\n"
"  ]             As the first character after any ^, matches itself.\n"
"                Otherwise, ends the character class.\n"
"\n"
"  [:alnum:]     [A-Za-z0-9]\n"
"  [:alpha:]     [A-Za-z]\n"
"  [:blank:]     Space or tab\n"
"  [:cntrl:]     Any control character\n"
"  [:digit:]     [0-9]\n"
"  [:graph:]     Any printable character except space\n"
"  [:lower:]     [a-z]\n"
"  [:print:]     Any printable character including space\n"
"  [:punct:]     Any printable character except [^ A-Za-z0-9]\n"
"  [:space:]     Space, tab, newline, carriage return, form feed, vertical tab\n"
"  [:upper:]     [A-Z]\n"
"  [:xdigit:]    Any hexadecimal digit: [0-9A-Fa-f]\n"
"\n"
"  [= =]         POSIX character equivalent\n"
"  [. .]         POSIX collating sequence\n"
"\n"
"  [[:&lt:]]       Pseudo-POSIX word beginning\n"
"  [[:&gt:]]       Pseudo-POSIX word ending\n"

"</font></pre>"
"</html>"
;
